#!/usr/bin/env swift

import AppKit
import Foundation
import IOKit.i2c

print("Displays:")
for screen in NSScreen.screens {
	let displayID = screen.deviceDescription[.init("NSScreenNumber")] as? CGDirectDisplayID ?? 0
	print("    ID: \(displayID)")
	print("        Vendor: \(CGDisplayVendorNumber(displayID))")
	print("        Model:  \(CGDisplayModelNumber(displayID))")
	print("        Serial: \(CGDisplaySerialNumber(displayID))")
	print("        Unit:   \(CGDisplayUnitNumber(displayID))")
}
print()

print("Framebuffers:")
var iter: io_iterator_t = 0
var serv: io_service_t = 0

if IOServiceGetMatchingServices(kIOMasterPortDefault, IOServiceMatching(IOFRAMEBUFFER_CONFORMSTO), &iter) == kIOReturnSuccess {
	defer { IOObjectRelease(iter) }
	while (serv = IOIteratorNext(iter), serv).1 != MACH_PORT_NULL {
		defer { IOObjectRelease(serv) }
		var busCount: IOItemCount = 0
		IOFBGetI2CInterfaceCount(serv, &busCount)
		if busCount >= 1 {
			let info = IODisplayCreateInfoDictionary(serv, IOOptionBits(kIODisplayNoProductName)).takeRetainedValue() as NSDictionary
			print("    Port: \(serv)")
			print("        Vendor:   \(info[kDisplayVendorID] ?? 0)")
			print("        Model:    \(info[kDisplayProductID] ?? 0)")
			print("        Serial:   \(info[kDisplaySerialNumber] ?? 0)")
			print("        Location: \(info[kIODisplayLocationKey] ?? "")")
		}
	}
}
